@@ -32,7 +32,7 @@ from xpra.os_util import (
     get_machine_id, get_user_uuid, register_SIGUSR_signals,
     filedata_nocrlf, force_quit,
     SIGNAMES, BITS,
-    strtobytes, bytestostr, hexstr, monotonic_time, use_tty,
+    strtobytes, bytestostr, hexstr, monotonic_time, use_gui_prompt,
     parse_encoded_bin_data,
     )
 from xpra.util import (
@@ -67,16 +67,16 @@ def noop():
     pass
 
 
-""" Base class for Xpra clients.
+class XpraClientBase(ServerInfoMixin, FilePrintMixin):
+    """
+    Base class for Xpra clients.
     Provides the glue code for:
     * sending packets via Protocol
     * handling packets received via _process_packet
     For an actual implementation, look at:
     * GObjectXpraClient
-    * xpra.client.gtk2.client
     * xpra.client.gtk3.client
-"""
-class XpraClientBase(ServerInfoMixin, FilePrintMixin):
+    """
 
     INSTALL_SIGNAL_HANDLERS = True
 
@@ -153,7 +153,9 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         self.encryption = opts.encryption or opts.tcp_encryption
         self.encryption_keyfile = opts.encryption_keyfile or opts.tcp_encryption_keyfile
         self.init_challenge_handlers(opts.challenge_handlers)
-        self.init_aliases()
+        #this is now done in UI client only,
+        #most simple clients are just wasting time doing this
+        #self.init_aliases()
         if self.INSTALL_SIGNAL_HANDLERS:
             self.install_signal_handlers()
 
@@ -700,8 +702,8 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
     #utility method used by some authentication handlers,
     #and overriden in UI client to provide a GUI dialog
     def do_process_challenge_prompt(self, packet, prompt="password"):
-        authlog("do_process_challenge_prompt() use_tty=%s", use_tty())
-        if use_tty():
+        authlog("do_process_challenge_prompt() use_gui_prompt=%s", use_gui_prompt())
+        if not use_gui_prompt():
             import getpass
             authlog("stdin isatty, using password prompt")
             password = getpass.getpass("%s :" % self.get_challenge_prompt(prompt))
@@ -915,12 +917,12 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         return True
 
 
-    def parse_server_capabilities(self, caps : typedict) -> bool:
-        for c in XpraClientBase.__bases__:
-            if not c.parse_server_capabilities(self, caps):
+    def parse_server_capabilities(self, c : typedict) -> bool:
+        for bc in XpraClientBase.__bases__:
+            if not bc.parse_server_capabilities(self, c):
                 return False
-        self.server_client_shutdown = caps.boolget("client-shutdown", True)
-        self.server_compressors = caps.strtupleget("compressors", ("zlib",))
+        self.server_client_shutdown = c.boolget("client-shutdown", True)
+        self.server_compressors = c.strtupleget("compressors", ("zlib",))
         return True
 
     def parse_network_capabilities(self, caps : typedict) -> bool:
@@ -958,30 +960,32 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
     def _process_gibberish(self, packet):
         log("process_gibberish(%s)", ellipsizer(packet))
         message, data = packet[1:3]
+        from xpra.net.socket_util import guess_packet_type
+        packet_type = guess_packet_type(data)
         p = self._protocol
-        show_as_text = p and p.input_packetcount==0 and len(data)<128 and all(c in string.printable for c in bytestostr(data))
-        if show_as_text:
-            #looks like the first packet back is just text, print it:
-            data = bytestostr(data)
-            if data.find("\n")>=0:
-                for x in data.splitlines():
-                    netlog.warn(x)
-            else:
-                netlog.error("Error: failed to connect, received")
-                netlog.error(" %s", repr_ellipsized(data))
+        pcount = p.input_packetcount==0 if p else 0
+        if pcount<=1:
+            netlog.error("Error: failed to connect")
         else:
-            from xpra.net.socket_util import guess_packet_type
-            packet_type = guess_packet_type(data)
-            log("guess_packet_type(%r)=%s", data, packet_type)
-            if packet_type and packet_type!="xpra":
-                netlog.error("Error: received a %s packet", packet_type)
-                netlog.error(" this is not an xpra server?")
+            netlog.error("Error: received an invalid packet")
+        if packet_type and packet_type=="xpra":
+            netlog.error(" xpra server bug or mangled packet")
+        if packet_type and packet_type!="xpra":
+            netlog.error(" this is a '%s' packet,", packet_type)
+            netlog.error(" not from an xpra server?")
+        else:
+            netlog.error(" received uninterpretable nonsense: %r", message)
+        data = bytestostr(data).strip("\n\r")
+        show_as_text = pcount<=1 and len(data)<128 and all((c in string.printable) or c in ("\n\r") for c in data)
+        if show_as_text:
+            if data.find("\n")>=0:
+                netlog.error(" data:")
+                for x in data.split("\n"):
+                    netlog.error("  %r", x.split("\0")[0])
             else:
-                netlog.error("Error: received uninterpretable nonsense: %s", message)
-            if p:
-                netlog.error(" packet no %i data: %s", p.input_packetcount, repr_ellipsized(data))
-            else:
-                netlog.error(" data: %s", repr_ellipsized(data))
+                netlog.error(" data: %r", data)
+        else:
+            netlog.error(" packet no %i data: %s", pcount, nonl(repr_ellipsized(data)))
         self.quit(EXIT_PACKET_FAILURE)
 
     def _process_invalid(self, packet):
