@@ -7,11 +7,12 @@
 
 import os
 import weakref
+from subprocess import Popen, PIPE
 from gi.repository import Gtk, Gdk, GdkPixbuf
 
 from xpra.client.gtk_base.gtk_client_window_base import HAS_X11_BINDINGS, XSHAPE
 from xpra.util import (
-    updict, pver, iround, flatten_dict,
+    updict, pver, iround, flatten_dict, noerr,
     envbool, envint, repr_ellipsized, csv, first_time, typedict,
     DEFAULT_METADATA_SUPPORTED, XPRA_OPENGL_NOTIFICATION_ID,
     )
@@ -22,6 +23,7 @@ from xpra.os_util import (
 from xpra.simple_stats import std_unit
 from xpra.exit_codes import EXIT_PASSWORD_REQUIRED
 from xpra.scripts.config import TRUE_OPTIONS, FALSE_OPTIONS
+from xpra.make_thread import start_thread
 from xpra.gtk_common.cursor_names import cursor_types
 from xpra.gtk_common.gtk_util import (
     get_gtk_version_info, scaled_image, get_default_cursor, color_parse,
@@ -83,6 +85,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def __init__(self):
         GObjectXpraClient.__init__(self)
         UIXpraClient.__init__(self)
+        self.pinentry_proc = None
         self.shortcuts_info = None
         self.session_info = None
         self.bug_report = None
@@ -181,6 +184,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def cleanup(self):
         log("GTKXpraClient.cleanup()")
+        self.stop_pinentry()
         if self.shortcuts_info:
             self.shortcuts_info.destroy()
             self.shortcuts_info = None
@@ -265,7 +269,54 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
 
     def do_process_challenge_prompt(self, packet, prompt="password"):
+        authlog = Logger("auth")
         self.show_progress(100, "authentication")
+        PINENTRY = os.environ.get("XPRA_PINENTRY", "")
+        from xpra.scripts.main import get_pinentry_command
+        pinentry_cmd = get_pinentry_command(PINENTRY)
+        authlog("do_process_challenge_prompt%s get_pinentry_command(%s)=%s",
+                (packet, prompt), PINENTRY, pinentry_cmd)
+        if pinentry_cmd:
+            start_thread(self.handle_challenge_with_pinentry,
+                         "pinentry", True, (packet, prompt, pinentry_cmd))
+            return True
+        return self.do_process_challenge_prompt_dialog(packet, prompt)
+
+    def stop_pinentry(self):
+        pp = self.pinentry_proc
+        if pp:
+            self.pinentry_proc = None
+            noerr(pp.terminate)
+
+
+    def handle_challenge_with_pinentry(self, packet, prompt="password", cmd="pinentry"):
+        authlog = Logger("auth")
+        authlog("handle_challenge_with_pinentry%s", (packet, prompt, cmd))
+        try:
+            proc = Popen([cmd], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+        except OSError:
+            authlog("pinentry failed", exc_info=True)
+        else:
+            self.pinentry_proc = proc
+            q = "Enter %s" % prompt
+            p = self._protocol
+            if p:
+                conn = getattr(p, "_conn", None)
+                if conn:
+                    from xpra.net.bytestreams import pretty_socket
+                    cinfo = conn.get_info()
+                    endpoint = pretty_socket(cinfo.get("endpoint", conn.target)).split("?")[0]
+                    q += " for %s" % endpoint
+            def got_pin(value):
+                self.idle_add(self.send_challenge_reply, packet, value)
+            def no_pin():
+                self.idle_add(self.quit, EXIT_PASSWORD_REQUIRED)
+            from xpra.scripts.main import pinentry_getpin
+            pinentry_getpin(proc, "Xpra Server Authentication:", q, got_pin, no_pin)
+            return True
+        return self.do_process_challenge_prompt_dialog(packet, prompt)
+
+    def do_process_challenge_prompt_dialog(self, packet, prompt="password"):
         dialog = Gtk.Dialog("Server Authentication",
                None,
                Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT)
@@ -989,7 +1040,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         screen.connect("size-changed", self.screen_size_changed)
 
 
-    def window_grab(self, window):
+    def window_grab(self, wid, window):
         em = Gdk.EventMask
         event_mask = (em.BUTTON_PRESS_MASK |
                       em.BUTTON_RELEASE_MASK |
@@ -999,16 +1050,20 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                       em.LEAVE_NOTIFY_MASK)
         confine_to = None
         cursor = None
-        r = Gdk.pointer_grab(window.get_window(), True, event_mask, confine_to, cursor, 0)
+        etime = Gtk.get_current_event_time()
+        r = Gdk.pointer_grab(window.get_window(), True, event_mask, confine_to, cursor, etime)
         grablog("pointer_grab(..)=%s", GRAB_STATUS_STRING.get(r, r))
         #also grab the keyboard so the user won't Alt-Tab away:
-        r = Gdk.keyboard_grab(window.get_window(), False, 0)
+        r = Gdk.keyboard_grab(window.get_window(), False, etime)
         grablog("keyboard_grab(..)=%s", GRAB_STATUS_STRING.get(r, r))
+        self._window_with_grab = wid
 
     def window_ungrab(self):
         grablog("window_ungrab()")
-        Gdk.pointer_ungrab(0)
-        Gdk.keyboard_ungrab(0)
+        etime = Gtk.get_current_event_time()
+        Gdk.pointer_ungrab(etime)
+        Gdk.keyboard_ungrab(etime)
+        self._window_with_grab = None
 
 
     def window_bell(self, window, device, percent, pitch, duration, bell_class, bell_id, bell_name):
@@ -1046,7 +1101,13 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def opengl_setup_failure(self, summary = "Xpra OpenGL Acceleration Failure", body=""):
         def delayed_notify():
             if self.exit_code is None:
-                self.may_notify(XPRA_OPENGL_NOTIFICATION_ID, summary, body, icon_name="opengl")
+                if OSX:
+                    #don't bother logging an error on MacOS,
+                    #OpenGL is being deprecated
+                    log.info(summary)
+                    log.info(body)
+                else:
+                    self.may_notify(XPRA_OPENGL_NOTIFICATION_ID, summary, body, icon_name="opengl")
         #wait for the main loop to run:
         self.timeout_add(2*1000, delayed_notify)
 
@@ -1176,6 +1237,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             elif self.client_supports_opengl:
                 opengllog("OpenGL supported with %s, but not enabled", driver_info)
             self.opengl_props["enabled"] = self.opengl_enabled
+            if self.opengl_enabled and not warnings and OSX:
+                #non-opengl is slow on MacOS:
+                self.opengl_force = True
         except ImportError as e:
             err("OpenGL accelerated rendering is not available:", e)
         except RuntimeError as e:
