@@ -17,7 +17,7 @@ from queue import Queue
 from gi.repository import GLib
 
 from xpra.platform.gui import (
-    get_vrefresh, get_window_min_size, get_window_max_size,
+    get_window_min_size, get_window_max_size,
     get_double_click_time, get_double_click_distance, get_native_system_tray_classes,
     )
 from xpra.platform.features import SYSTEM_TRAY_SUPPORTED
@@ -66,7 +66,6 @@ for s in OR_FORCE_GRAB_STR.split(","):
         OR_FORCE_GRAB.setdefault(parts[0], []).append(parts[1])
 
 SKIP_DUPLICATE_BUTTON_EVENTS = envbool("XPRA_SKIP_DUPLICATE_BUTTON_EVENTS", True)
-REVERSE_HORIZONTAL_SCROLLING = envbool("XPRA_REVERSE_HORIZONTAL_SCROLLING", OSX)
 
 DYNAMIC_TRAY_ICON = envbool("XPRA_DYNAMIC_TRAY_ICON", not OSX and not is_Ubuntu())
 ICON_OVERLAY = envint("XPRA_ICON_OVERLAY", 50)
@@ -309,7 +308,6 @@ class WindowClient(StubClientMixin):
             "double_click.distance"     : get_double_click_distance(),
             #features:
             "bell"                      : self.client_supports_bell,
-            "vrefresh"                  : self.get_vrefresh(),
             "windows"                   : self.windows_enabled,
             "auto_refresh_delay"        : int(self.auto_refresh_delay*1000),
             #system tray forwarding:
@@ -322,9 +320,6 @@ class WindowClient(StubClientMixin):
             })
         return caps
 
-    def get_vrefresh(self):
-        return get_vrefresh()
-
     def get_window_caps(self) -> dict:
         return {
             #implemented in the gtk client:
@@ -417,8 +412,6 @@ class WindowClient(StubClientMixin):
         #this is a different entry point for mouse wheel events,
         #which provides finer grained deltas (if supported by the server)
         #accumulate deltas:
-        if REVERSE_HORIZONTAL_SCROLLING:
-            deltax = -deltax
         self.wheel_deltax += deltax
         self.wheel_deltay += deltay
         button = self.wheel_map.get(6+int(self.wheel_deltax>0))            #RIGHT=7, LEFT=6
@@ -750,7 +743,7 @@ class WindowClient(StubClientMixin):
 
     def make_new_window(self, wid, wx, wy, ww, wh, bw, bh, metadata, override_redirect, client_properties):
         client_window_classes = self.get_client_window_classes(ww, wh, metadata, override_redirect)
-        group_leader_window = self.get_group_leader(wid, metadata, override_redirect)
+        group_leader_window = self.get_group_leader(wid, metadata, override_redirect)  #pylint: disable=assignment-from-none
         #workaround for "popup" OR windows without a transient-for (like: google chrome popups):
         #prevents them from being pushed under other windows on OSX
         #find a "transient-for" value using the pid to find a suitable window
@@ -793,7 +786,7 @@ class WindowClient(StubClientMixin):
         if override_redirect:
             if self.should_force_grab(metadata):
                 grablog.warn("forcing grab for OR window %i, matches %s", wid, OR_FORCE_GRAB)
-                self.window_grab(window)
+                self.window_grab(wid, window)
         return window
 
     def should_force_grab(self, metadata):
@@ -825,7 +818,7 @@ class WindowClient(StubClientMixin):
             from xpra.child_reaper import getChildReaper
             from subprocess import Popen, PIPE, STDOUT
             try:
-                proc = Popen("xpra_signal_listener",
+                proc = Popen(["xpra_signal_listener"],
                              stdin=PIPE, stdout=PIPE, stderr=STDOUT,
                              start_new_session=True)
             except OSError as e:
@@ -1257,7 +1250,9 @@ class WindowClient(StubClientMixin):
         else:
             if self._window_with_grab:
                 self.window_ungrab()
-                self.do_force_ungrab(self._window_with_grab)
+                wwgrab = self._window_with_grab
+                if wwgrab:
+                    self.do_force_ungrab(wwgrab)
                 self._window_with_grab = None
             if wid and self._focused and self._focused!=wid:
                 #if this window lost focus, it must have had it!
@@ -1287,11 +1282,13 @@ class WindowClient(StubClientMixin):
 
     ######################################################################
     # grabs:
-    def window_grab(self, _window):
+    def window_grab(self, wid, _window):
         grablog.warn("Warning: window grab not implemented in %s", self.client_type())
+        self._window_with_grab = wid
 
     def window_ungrab(self):
         grablog.warn("Warning: window ungrab not implemented in %s", self.client_type())
+        self._window_with_grab = None
 
     def do_force_ungrab(self, wid):
         grablog("do_force_ungrab(%s)", wid)
@@ -1303,15 +1300,13 @@ class WindowClient(StubClientMixin):
         window = self._id_to_window.get(wid)
         grablog("grabbing %s: %s", wid, window)
         if window:
-            self.window_grab(window)
-            self._window_with_grab = wid
+            self.window_grab(wid, window)
 
     def _process_pointer_ungrab(self, packet):
         wid = packet[1]
         window = self._id_to_window.get(wid)
         grablog("ungrabbing %s: %s", wid, window)
         self.window_ungrab()
-        self._window_with_grab = None
 
 
     ######################################################################
