@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
 # Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
@@ -8,6 +8,7 @@
 #pylint: disable=no-member
 
 import os
+from threading import RLock
 import pycuda               #@UnresolvedImport
 from pycuda import driver   #@UnresolvedImport
 
@@ -300,19 +301,18 @@ def select_device(preferred_device_id=-1, min_compute=0):
     log("select_device(%s, %s)", preferred_device_id, min_compute)
     for device_id in (preferred_device_id, get_pref("device-id")):
         if device_id is not None and device_id>=0:
-            #try to honour the device specified:
-            try:
-                device, context, tpct = load_device(device_id)
-            finally:
+            dct = load_device(device_id)
+            if dct:
+                device, context, tpct = dct
                 context.pop()
                 context.detach()
-            if min_compute>0:
-                compute = compute_capability(device)
-                if compute<min_compute:
-                    log.warn("Warning: GPU device %i only supports compute %#x", device_id, compute)
-            if tpct<MIN_FREE_MEMORY:
-                log.warn("Warning: GPU device %i is low on memory: %i%%", device_id, tpct)
-            return device_id, device
+                if min_compute>0:
+                    compute = compute_capability(device)
+                    if compute<min_compute:
+                        log.warn("Warning: GPU device %i only supports compute %#x", device_id, compute)
+                if tpct<MIN_FREE_MEMORY:
+                    log.warn("Warning: GPU device %i is low on memory: %i%%", device_id, tpct)
+                return device_id, device
     load_balancing = get_pref("load-balancing")
     log("load-balancing=%s", load_balancing)
     if load_balancing=="round-robin":
@@ -363,8 +363,11 @@ def select_best_free_memory(min_compute=0):
         log("will test %s device%s from %s list: %s", len(device_list), engs(device_list), list_name, device_list)
         for device_id in device_list:
             context = None
+            dct = load_device(device_id)
+            if not dct:
+                continue
             try:
-                device, context, tpct = load_device(device_id)
+                device, context, tpct = dct
                 compute = compute_capability(device)
                 if compute<min_compute:
                     log("ignoring device %s: compute capability %#x (minimum %#x required)",
@@ -392,10 +395,23 @@ def select_best_free_memory(min_compute=0):
 
 def load_device(device_id):
     log("load_device(%i)", device_id)
-    device = driver.Device(device_id)
+    try:
+        device = driver.Device(device_id)
+    except Exception as e:
+        log("load_device(%s)", device_id, exc_info=True)
+        log.error("Error: allocating CUDA device %s", device_id)
+        log.error(" %s", e)
+        return None
     log("select_device: testing device %s: %s", device_id, device_info(device))
     cf = driver.ctx_flags
-    context = device.make_context(flags=cf.SCHED_YIELD | cf.MAP_HOST)
+    flags = cf.SCHED_YIELD | cf.MAP_HOST
+    try:
+        context = device.make_context(flags=flags)
+    except Exception as e:
+        log("%s.make_context(%s)", flags, exc_info=True)
+        log.error("Error: cannot create CUDA context for device %s", device_id)
+        log.error(" %s", e)
+        return None
     log("created context=%s", context)
     free, total = driver.mem_get_info()
     log("memory: free=%sMB, total=%sMB",  int(free/1024/1024), int(total/1024/1024))
@@ -403,6 +419,63 @@ def load_device(device_id):
     return device, context, tpct
 
 
+def get_device_context(options):
+    MIN_COMPUTE = 0x30
+    device_id, device = select_device(options.intget("cuda_device", -1), min_compute=MIN_COMPUTE)
+    if device_id<0 or not device:
+        return None
+    return cuda_device_context(device_id, device)
+
+
+class cuda_device_context:
+    def __init__(self, device_id, device):
+        self.device_id = device_id
+        self.device = device
+        self.context = None
+        self.lock = RLock()
+        log("%r", self)
+
+    def __bool__(self):
+        return self.device is not None
+
+    def __enter__(self):
+        assert self.lock.acquire(False), "failed to acquire cuda device lock"
+        if not self.context:
+            start = monotonic_time()
+            cf = driver.ctx_flags
+            self.context = self.device.make_context(flags=cf.SCHED_YIELD | cf.MAP_HOST)
+            end = monotonic_time()
+            self.context.pop()
+            log("cuda context allocation took %ims", 1000*(end-start))
+        self.context.push()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        c = self.context
+        if c:
+            c.pop()
+        self.lock.release()
+
+    def __repr__(self):
+        return "cuda_device_context(%i - %s)" % (self.device_id, self.lock._is_owned())
+
+    def get_info(self):
+        return {
+            "id" : self.device_id,
+            }
+
+    def __del__(self):
+        self.free()
+
+    def free(self):
+        log("free() context=%s", self.context)
+        c = self.context
+        if c:
+            self.device_id = 0
+            self.device = None
+            self.context = None
+            with self.lock:
+                c.detach()
+
 
 CUDA_ERRORS_INFO = {
     #this list is taken from the CUDA 7.0 SDK header file,
