@@ -31,6 +31,7 @@ WAIT_PROBE_TIMEOUT = envint("XPRA_WAIT_PROBE_TIMEOUT", 6)
 GROUP = os.environ.get("XPRA_GROUP", "xpra")
 PEEK_TIMEOUT = envint("XPRA_PEEK_TIMEOUT", 1)
 PEEK_TIMEOUT_MS = envint("XPRA_PEEK_TIMEOUT_MS", PEEK_TIMEOUT*1000)
+UNIXDOMAIN_PEEK_TIMEOUT_MS = envint("XPRA_UNIX_DOMAIN_PEEK_TIMEOUT_MS", 100)
 PEEK_SIZE = envint("XPRA_PEEK_SIZE", 8192)
 
 SOCKET_DIR_MODE = num = int(os.environ.get("XPRA_SOCKET_DIR_MODE", "775"), 8)
@@ -285,6 +286,8 @@ def guess_packet_type(data):
         return "ssh"
     if data[0]==0x16:
         return "ssl"
+    if data[:4]==b"RFB ":
+        return "vnc"
     line1 = data.splitlines()[0]
     if line1.find(b"HTTP/")>0 or line1.split(b" ")[0] in (b"GET", b"POST"):
         return "http"
