@@ -11,7 +11,11 @@ import signal
 from time import sleep
 from subprocess import PIPE, Popen
 
-from xpra.scripts.main import InitException, InitExit, shellquote, host_target_string
+from xpra.scripts.main import (
+    InitException, InitExit,
+    shellquote, host_target_string,
+    get_pinentry_command, run_pinentry_getpin, run_pinentry_confirm,
+    )
 from xpra.platform.paths import get_ssh_known_hosts_files
 from xpra.platform import get_username
 from xpra.scripts.config import parse_bool
@@ -24,6 +28,7 @@ from xpra.exit_codes import (
 from xpra.os_util import (
     bytestostr, osexpand, load_binary_file, monotonic_time,
     nomodule_context, umask_context, is_main_thread,
+    use_gui_prompt,
     WIN32, OSX, POSIX,
     )
 from xpra.util import envint, envbool, envfloat, engs, csv
@@ -35,6 +40,7 @@ INITENV_COMMAND = os.environ.get("XPRA_INITENV_COMMAND", "")    #"xpra initenv"
 WINDOW_SIZE = envint("XPRA_SSH_WINDOW_SIZE", 2**27-1)
 TIMEOUT = envint("XPRA_SSH_TIMEOUT", 60)
 SKIP_UI = envbool("XPRA_SKIP_UI", False)
+PINENTRY = envbool("XPRA_SSH_PINENTRY", POSIX and not OSX)
 
 VERIFY_HOSTKEY = envbool("XPRA_SSH_VERIFY_HOSTKEY", True)
 VERIFY_STRICT = envbool("XPRA_SSH_VERIFY_STRICT", False)
@@ -96,6 +102,10 @@ def dialog_run(dialog) -> int:
     return code[0]
 
 def dialog_pass(title="Password Input", prompt="enter password", icon="") -> str:
+    if PINENTRY:
+        pinentry_cmd = get_pinentry_command()
+        if pinentry_cmd:
+            return run_pinentry_getpin(pinentry_cmd, title, prompt)
     from xpra.client.gtk_base.pass_dialog import PasswordInputDialogWindow
     dialog = PasswordInputDialogWindow(title, prompt, icon)
     try:
@@ -115,15 +125,18 @@ def dialog_confirm(title, prompt, qinfo=(), icon="", buttons=(("OK", 1),)) -> in
     return r
 
 
-def confirm_key(info=()) -> bool:
+def confirm_key(info=(), title="Confirm Key", prompt="Are you sure you want to continue connecting?") -> bool:
     if SKIP_UI:
         return False
     from xpra.platform.paths import get_icon_filename
-    from xpra.os_util import use_tty
-    if not use_tty():
+    if PINENTRY:
+        pinentry_cmd = get_pinentry_command()
+        if pinentry_cmd:
+            messages = list(info)+["", prompt]
+            return run_pinentry_confirm(pinentry_cmd, title, "%0A".join(messages))
+    if use_gui_prompt():
         icon = get_icon_filename("authentication", "png") or ""
-        prompt = "Are you sure you want to continue connecting?"
-        code = dialog_confirm("Confirm Key", prompt, info, icon, buttons=[("yes", 200), ("NO", 201)])
+        code = dialog_confirm(title, prompt, info, icon, buttons=[("yes", 200), ("NO", 201)])
         log("dialog return code=%s", code)
         r = code==200
         log.info("host key %sconfirmed", ["not ", ""][r])
@@ -141,8 +154,7 @@ def input_pass(prompt) -> str:
     if SKIP_UI:
         return None
     from xpra.platform.paths import get_icon_filename
-    from xpra.os_util import use_tty
-    if not use_tty():
+    if PINENTRY or use_gui_prompt():
         icon = get_icon_filename("authentication", "png") or ""
         return dialog_pass("Password Input", prompt, icon)
     from getpass import getpass
@@ -355,7 +367,7 @@ def ssh_paramiko_connect_to(display_desc):
                 transport.start_client()
             except SSHException as e:
                 log("start_client()", exc_info=True)
-                raise InitExit(EXIT_SSH_FAILURE, "SSH negotiation failed: %s" % e)
+                raise InitExit(EXIT_SSH_FAILURE, "SSH negotiation failed: %s" % e) from None
             do_ssh_paramiko_connect_to(transport, host,
                                        username, password,
                                        host_config or ssh_config.lookup("*"),
@@ -514,7 +526,7 @@ keymd5(host_key),
             else:
                 assert (not keys) or (host_key.get_name() not in keys)
                 if not keys:
-                    log.warn("Warning: unknown SSH host")
+                    log.warn("Warning: unknown SSH host '%s'", host)
                 else:
                     log.warn("Warning: unknown %s SSH host key", keyname())
                 qinfo = [
@@ -572,7 +584,8 @@ keymd5(host_key),
                         log("authenticated using agent and key '%s'", keymd5(agent_key))
                         break
                 except SSHException:
-                    log("agent key '%s' rejected", keymd5(agent_key), exc_info=True)
+                    log.info("SSH agent key '%s' rejected for user '%s'", keymd5(agent_key), username)
+                    log("%s%s", transport.auth_publickey, (username, agent_key), exc_info=True)
             if not transport.is_authenticated():
                 log.info("agent authentication failed, tried %i key%s", len(agent_keys), engs(agent_keys))
 
@@ -585,7 +598,14 @@ keymd5(host_key),
             log("trying '%s'", keyfile_path)
             key = None
             import paramiko
-            for pkey_classname in ("RSA", "DSS", "ECDSA", "Ed25519"):
+            try_key_formats = ()
+            for kf in ("RSA", "DSS", "ECDSA", "Ed25519"):
+                if keyfile_path.lower().endswith(kf.lower()):
+                    try_key_formats = (kf, )
+                    break
+            if not try_key_formats:
+                try_key_formats = ("RSA", "DSS", "ECDSA", "Ed25519")
+            for pkey_classname in try_key_formats:
                 pkey_class = getattr(paramiko, "%sKey" % pkey_classname, None)
                 if pkey_class is None:
                     log("no %s key type", pkey_classname)
@@ -784,7 +804,6 @@ def paramiko_run_remote_xpra(transport, xpra_proxy_command=None, remote_xpra=Non
                 installpath = bytestostr(lines[-1])
                 xpra_cmd = "%s\\%s" % (installpath, xpra_cmd)
                 xpra_cmd = xpra_cmd.replace("\\", "\\\\")
-                log("using '%s'", xpra_cmd)
         elif xpra_cmd.endswith(".exe"):
             #assume this path exists
             pass
@@ -796,9 +815,10 @@ def paramiko_run_remote_xpra(transport, xpra_proxy_command=None, remote_xpra=Non
             if r[0]:
                 #use the actual path returned by 'which':
                 try:
-                    xpra_cmd = r[0].decode().rstrip("\n\r")
-                except:
+                    xpra_cmd = r[0].decode().splitlines()[-1].rstrip("\n\r ").lstrip("\t ")
+                except Exception:
                     pass
+        log("adding xpra_cmd='%s'", xpra_cmd)
         if xpra_cmd in tried:
             continue
         tried.add(xpra_cmd)
