@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2013-2021 Antoine Martin <antoine@xpra.org>
@@ -207,7 +207,7 @@ def platform_release(release):
     return release
 
 
-def platform_name(sys_platform, release=None) -> str:
+def platform_name(sys_platform=sys.platform, release=None) -> str:
     if not sys_platform:
         return "unknown"
     PLATFORMS = {"win32"    : "Microsoft Windows",
@@ -568,6 +568,9 @@ def use_tty():
     from xpra.platform.gui import use_stdin
     return use_stdin()
 
+def use_gui_prompt():
+    return WIN32 or OSX or not use_tty()
+
 
 def shellsub(s, subs=None):
     """ shell style string substitution using the dictionary given """
