@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
 # Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
@@ -7,7 +7,7 @@
 import sys
 import os
 
-from xpra.util import csv
+from xpra.util import csv, sorted_nicely
 from xpra.os_util import (
     WIN32, OSX, POSIX,
     osexpand, getuid, getgid, get_username_for_uid,
@@ -105,7 +105,7 @@ def get_Xdummy_command(xorg_cmd="Xorg", log_dir="${XPRA_LOG_DIR}", xorg_conf="/e
           "-logfile", "%s/Xorg.${DISPLAY}.log" % log_dir,
           #must be specified with some Xorg versions (ie: arch linux)
           #this directory can store xorg config files, it does not need to be created:
-          "-configdir", get_Xdummy_confdir(),
+          "-configdir", '"%s"' % get_Xdummy_confdir(),
           "-config", xorg_conf
           ]
     return cmd
@@ -125,15 +125,15 @@ def get_Xvfb_command(width=8192, height=4096, dpi=96):
     return cmd
 
 def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None,
-                        Xdummy_ENABLED=None, Xdummy_wrapper_ENABLED=None, warn=warn):
+                        Xdummy_ENABLED=None, Xdummy_wrapper_ENABLED=None, warn_fn=warn):
     #returns the xvfb command to use
     if WIN32:   # pragma: no cover
         return ""
     if OSX:     # pragma: no cover
         return get_Xvfb_command()
     if sys.platform.find("bsd")>=0 and Xdummy_ENABLED is None:  # pragma: no cover
-        if warn:
-            warn("Warning: sorry, no support for Xdummy on %s" % sys.platform)
+        if warn_fn:
+            warn_fn("Warning: sorry, no support for Xdummy on %s" % sys.platform)
         return get_Xvfb_command()
     if is_arm():
         #arm struggles to launch Xdummy, so use Xvfb:
@@ -149,8 +149,8 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None,
             #honour what was specified:
             use_wrapper = Xdummy_wrapper_ENABLED
         elif not xorg_bin:
-            if warn:
-                warn("Warning: Xorg binary not found, assuming the wrapper is needed!")
+            if warn_fn:
+                warn_fn("Warning: Xorg binary not found, assuming the wrapper is needed!")
             use_wrapper = True
         else:
             #auto-detect
@@ -158,8 +158,8 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None,
             xorg_stat = os.stat(xorg_bin)
             if (xorg_stat.st_mode & stat.S_ISUID)!=0:
                 if (xorg_stat.st_mode & stat.S_IROTH)==0:
-                    if warn:
-                        warn("%s is suid and not readable, Xdummy support unavailable" % xorg_bin)
+                    if warn_fn:
+                        warn_fn("%s is suid and not readable, Xdummy support unavailable" % xorg_bin)
                     return get_Xvfb_command()
                 debug("%s is suid and readable, using the xpra_Xdummy wrapper" % xorg_bin)
                 use_wrapper = True
@@ -542,6 +542,7 @@ OPTION_TYPES = {
                     "speaker"           : str,
                     "sound-source"      : str,
                     "html"              : str,
+                    "http-scripts"      : str,
                     "socket-permissions": str,
                     "exec-wrapper"      : str,
                     "dbus-launch"       : str,
@@ -605,7 +606,7 @@ OPTION_TYPES = {
                     "cursors"           : bool,
                     "bell"              : bool,
                     "notifications"     : bool,
-                    "xsettings"         : bool,
+                    "xsettings"         : str,
                     "system-tray"       : bool,
                     "sharing"           : bool,
                     "lock"              : bool,
@@ -763,7 +764,7 @@ PROXY_START_OVERRIDABLE_OPTIONS = [
     "use-display",
     "fake-xinerama", "resize_display", "dpi", "pixel-depth",
     "readonly", "keyboard-sync", "cursors", "bell", "notifications", "xsettings",
-    "system-tray", "sharing", "lock", "windows", "webcam", "html",
+    "system-tray", "sharing", "lock", "windows", "webcam", "html", "http-scripts",
     "terminate-children", "exit-with-children", "exit-with-client",
     "av-sync", "global-menus",
     "forward-xdg-open", "modal-windows", "bandwidth-detection",
@@ -883,13 +884,18 @@ def get_defaults():
     for conf_dir in conf_dirs:
         if conf_dir and os.path.exists(conf_dir):
             break
-    xvfb = detect_xvfb_command(conf_dir, bin_dir, warn=None)
+    xvfb = detect_xvfb_command(conf_dir, bin_dir, warn_fn=None)
     xvfb_str = xvfb_cmd_str(xvfb)
 
     ssl_protocol = "TLSv1_2"
 
     if POSIX and not OSX and not (is_Debian() or is_Ubuntu()):
-        fake_xinerama = "auto"
+        #try to find it quickly:
+        try:
+            import glob
+            fake_xinerama = sorted_nicely(glob.glob("/usr/lib*/libfakeXinerama.so.*"))[-1]
+        except Exception:
+            fake_xinerama = "auto"
     else:
         fake_xinerama = "no"
 
@@ -934,7 +940,7 @@ def get_defaults():
                     "window-close"      : "auto",
                     "min-size"          : "",
                     "max-size"          : "",
-                    "desktop-scaling"   : "auto",
+                    "desktop-scaling"   : "on",
                     "display"           : "",
                     "tcp-proxy"         : "",
                     "download-path"     : get_download_dir(),
@@ -942,18 +948,19 @@ def get_defaults():
                     "remote-logging"    : "both",
                     "lpadmin"           : "/usr/sbin/lpadmin",
                     "lpinfo"            : "/usr/sbin/lpinfo",
-                    "add-printer-options" : ["-E", "-o printer-is-shared=false", "-u allow:$USER"],
+                    "add-printer-options" : ["-u allow:$USER", "-E", "-o printer-is-shared=false"],
                     "pdf-printer"       : "",
                     "postscript-printer": DEFAULT_POSTSCRIPT_PRINTER,
                     "debug"             : "",
                     "input-method"      : "none",
                     "sound-source"      : "",
                     "html"              : "auto",
+                    "http-scripts"      : "all",
                     "socket-permissions": "600",
                     "exec-wrapper"      : "",
                     "dbus-launch"       : "dbus-launch --sh-syntax --close-stderr",
                     "webcam"            : ["auto", "no"][OSX or WIN32],
-                    "mousewheel"        : "on",
+                    "mousewheel"        : ["on", "invert-x"][OSX],
                     "input-devices"     : "auto",
                     "shortcut-modifiers": "auto",
                     "open-files"        : "auto",
@@ -1013,7 +1020,7 @@ def get_defaults():
                     "cursors"           : True,
                     "bell"              : True,
                     "notifications"     : True,
-                    "xsettings"         : not OSX and not WIN32,
+                    "xsettings"         : ["auto", "no"][int(OSX or WIN32)],
                     "system-tray"       : True,
                     "sharing"           : None,
                     "lock"              : None,
@@ -1442,10 +1449,18 @@ def fixup_clipboard(options):
 
 def abs_paths(options):
     ew = options.exec_wrapper
-    if ew and not os.path.isabs(ew):
-        ew = which(ew)
-        if ew:
-            options.exec_wrapper = ew
+    if ew:
+        import shlex
+        ewp = shlex.split(ew)
+        if ewp and not os.path.isabs(ewp[0]):
+            abscmd = which(ewp[0])
+            if abscmd:
+                ewp[0] = abscmd
+                try:
+                    options.exec_wrapper = shlex.join(ewp)
+                except AttributeError:
+                    #(shlex.join requires Python 3.8)
+                    options.exec_wrapper = " ".join(shlex.quote(x) for x in ewp)
     #convert to absolute paths before we daemonize
     for k in ("clipboard-filter-file",
               "tcp-encryption-keyfile", "encryption-keyfile",
