@@ -12,9 +12,15 @@
 import sys
 import os.path
 import atexit
+import datetime
 import traceback
 
-from xpra.scripts.main import info, warn, no_gtk, validate_encryption, parse_env, configure_env
+from xpra.scripts.main import (
+    info, warn,
+    no_gtk, bypass_no_gtk,
+    validate_encryption, parse_env, configure_env,
+    stat_X11_display, get_xpra_sessions,
+    )
 from xpra.scripts.config import InitException, FALSE_OPTIONS, parse_bool
 from xpra.common import CLOBBER_USE_DISPLAY, CLOBBER_UPGRADE
 from xpra.exit_codes import EXIT_VFB_ERROR, EXIT_OK
@@ -22,6 +28,7 @@ from xpra.os_util import (
     SIGNAMES, POSIX, WIN32, OSX,
     FDChangeCaptureContext,
     force_quit,
+    get_rand_chars,
     get_username_for_uid, get_home_for_uid, get_shell_for_uid, getuid, setuidgid,
     get_hex_uuid, get_status_output, strtobytes, bytestostr, get_util_logger, osexpand,
     )
@@ -91,10 +98,11 @@ def _get_str(prop_name):
 
 
 def save_xvfb_pid(pid):
-    _save_int("_XPRA_SERVER_PID", pid)
+    _save_int("XPRA_XVFB_PID", pid)
 
 def get_xvfb_pid():
-    return _get_int("_XPRA_SERVER_PID")
+    #lookup the legacy name too:
+    return _get_int("XPRA_XVFB_PID") or _get_int("_XPRA_SERVER_PID")
 
 
 def save_uinput_id(uuid):
@@ -321,9 +329,9 @@ def set_server_features(opts):
     server_features.rfb             = b(opts.rfb_upgrade) and impcheck("server.rfb")
 
 
-def make_desktop_server(clobber):
+def make_desktop_server():
     from xpra.x11.desktop_server import XpraDesktopServer
-    return XpraDesktopServer(clobber)
+    return XpraDesktopServer()
 
 def make_server(clobber):
     from xpra.x11.server import XpraServer
@@ -416,18 +424,16 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         warn(" but 'exit-with-children' is not enabled,")
         warn(" use 'start' instead")
 
-    if opts.bind_rfb and (proxying or starting):
-        get_util_logger().warn("Warning: bind-rfb sockets cannot be used with '%s' mode" % mode)
-        opts.bind_rfb = []
-
-    if not shadowing and not starting_desktop:
+    if not (shadowing or starting_desktop or upgrading_desktop):
         opts.rfb_upgrade = 0
+        if opts.bind_rfb:
+            get_util_logger().warn("Warning: bind-rfb sockets cannot be used with '%s' mode" % mode)
+            opts.bind_rfb = []
 
-    if upgrading or upgrading_desktop or shadowing:
+    if (upgrading or upgrading_desktop or shadowing) and opts.pulseaudio is None:
         #there should already be one running
         #so change None ('auto') to False
-        if opts.pulseaudio is None:
-            opts.pulseaudio = False
+        opts.pulseaudio = False
 
     #get the display name:
     if shadowing and not extra_args:
@@ -470,6 +476,31 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
                 # Use the temporary magic value 'S' as marker:
                 display_name = 'S' + str(os.getpid())
 
+    if upgrading or upgrading_desktop:
+        assert display_name, "no display found to upgrade"
+        #try to stop the existing server if it exists:
+        dotxpra = DotXpra(opts.socket_dir, opts.socket_dirs)
+        sessions = get_xpra_sessions(dotxpra, ignore_state=(DotXpra.UNKNOWN, DotXpra.DEAD), matching_display=display_name, query=True)
+        session = sessions.get(display_name)
+        if session:
+            from xpra.platform.paths import get_xpra_command
+            cmd = get_xpra_command()+["exit"]
+            socket_path = session.get("socket-path")
+            if socket_path:
+                cmd += ["socket://%s" % socket_path]
+            else:
+                cmd += [display_name]
+            env = os.environ.copy()
+            #don't wait too long:
+            env["XPRA_CONNECT_TIMEOUT"] = "5"
+            from subprocess import Popen  #pylint: disable=import-outside-toplevel
+            try:
+                p = Popen(cmd, env=env)
+                p.wait()
+            except OSError as e:
+                noerr(sys.stderr.write, "Error: failed to 'exit' the server to upgrade\n")
+                noerr(sys.stderr.write, " %s\n" % e)
+
     if not (shadowing or proxying or upgrading or upgrading_desktop) and \
     opts.exit_with_children and not has_child_arg:
         error_cb("--exit-with-children specified without any children to spawn; exiting immediately")
@@ -590,7 +621,6 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         # environment:
         write_runner_shell_scripts(script)
 
-    import datetime
     extra_expand = {"TIMESTAMP" : datetime.datetime.now().strftime("%Y%m%d-%H%M%S")}
     log_to_file = opts.daemon or os.environ.get("XPRA_LOG_TO_FILE", "")=="1"
     if start_vfb or log_to_file:
@@ -673,34 +703,82 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
     xvfb = None
     xvfb_pid = None
     uinput_uuid = None
-    if start_vfb and use_display is None:
-        #use-display='auto' so we have to figure out
-        #if we have to start the vfb or not:
-        if not display_name:
-            use_display = False
+    xauthority = None
+    if start_vfb or clobber:
+        #XAUTHORITY
+        from xpra.x11.vfb_util import (
+            start_Xvfb, check_xvfb_process, parse_resolution,
+            get_xauthority_path,
+            xauth_add,
+            )
+        xauthority = get_xauthority_path(display_name, username, uid, gid)
+        os.environ["XAUTHORITY"] = xauthority
+        if not os.path.exists(xauthority):
+            log("creating XAUTHORITY file '%s'", xauthority)
+            try:
+                with open(xauthority, "a") as f:
+                    if getuid()==0 and (uid!=0 or gid!=0):
+                        os.fchown(f.fileno(), uid, gid)
+            except Exception as e:
+                #trying to continue anyway!
+                log.error("Error trying to create XAUTHORITY file %s:", xauthority)
+                log.error(" %s", e)
         else:
-            progress(20, "connecting to the display")
-            if verify_display(None, display_name, log_errors=False, timeout=1)==0:
-                start_vfb = False
+            log("found existing XAUTHORITY file '%s'", xauthority)
+        #resolve use-display='auto':
+        if use_display is None or upgrading or upgrading_desktop:
+            #figure out if we have to start the vfb or not:
+            if not display_name:
+                if upgrading or upgrading_desktop:
+                    error_cb("no displays found to upgrade")
+                use_display = False
+            else:
+                progress(20, "connecting to the display")
+                display_no = int(display_name[1:])
+                stat = stat_X11_display(display_no)
+                log("stat_X11_display(%i)=%s", display_no, stat)
+                if not stat:
+                    if upgrading or upgrading_desktop:
+                        error_cb("cannot access display '%s'" % (display_name,))
+                    #no X11 socket to connect to, so we have to start one:
+                    start_vfb = True
+                elif verify_display(None, display_name, log_errors=False, timeout=1)==0:
+                    #accessed OK:
+                    start_vfb = False
+                    #we have already loaded gtk in 'verify_display':
+                    bypass_no_gtk()
+                else:
+                    #verify failed but we can stat the X11 server socket...
+                    #perhaps we need to re-add an xauth entry
+                    if not xauth_data:
+                        xauth_data = get_hex_uuid()
+                        if pam:
+                            pam.set_items({"XAUTHDATA" : xauth_data})
+                    xauth_add(xauthority, display_name, xauth_data, uid, gid)
+                    if verify_display(None, display_name, log_errors=False, timeout=1)!=0:
+                        warn("display %s is not accessible" % (display_name,))
+                    else:
+                        #now OK!
+                        start_vfb = False
     if start_vfb:
         progress(20, "starting a virtual display")
         assert not proxying and xauth_data
         pixel_depth = validate_pixel_depth(opts.pixel_depth, starting_desktop)
-        from xpra.x11.vfb_util import start_Xvfb, check_xvfb_process, parse_resolution
         from xpra.server.server_util import has_uinput
         uinput_uuid = None
         if has_uinput() and opts.input_devices.lower() in ("uinput", "auto") and not shadowing:
-            from xpra.os_util import get_rand_chars
             uinput_uuid = get_rand_chars(UINPUT_UUID_LEN)
         vfb_geom = ""
         try:
             vfb_geom = parse_resolution(opts.resize_display)
         except Exception:
             pass
+
         xvfb, display_name, cleanups = start_Xvfb(opts.xvfb, vfb_geom, pixel_depth, display_name, cwd,
-                                                  uid, gid, username, xauth_data, uinput_uuid)
+                                                  uid, gid, username, uinput_uuid)
         for f in cleanups:
             add_cleanup(f)
+        xauth_add(xauthority, display_name, xauth_data, uid, gid)
         xvfb_pid = xvfb.pid
         #always update as we may now have the "real" display name:
         os.environ["DISPLAY"] = display_name
@@ -709,19 +787,10 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         if display_name!=odisplay_name and pam:
             pam.set_items({"XDISPLAY" : display_name})
 
-        def check_xvfb(timeout=0):
-            return check_xvfb_process(xvfb, timeout=timeout, command=opts.xvfb)
-    else:
-        if POSIX and clobber:
-            #if we're meant to be using a private XAUTHORITY file,
-            #make sure to point to it:
-            from xpra.x11.vfb_util import get_xauthority_path
-            xauthority = get_xauthority_path(display_name, username, uid, gid)
-            if os.path.exists(xauthority):
-                log("found XAUTHORITY=%s", xauthority)
-                os.environ["XAUTHORITY"] = xauthority
-        def check_xvfb(timeout=0):
+    def check_xvfb(timeout=0):
+        if xvfb is None:
             return True
+        return check_xvfb_process(xvfb, timeout=timeout, command=opts.xvfb)
 
     if POSIX and not OSX and displayfd>0:
         from xpra.platform.displayfd import write_displayfd
@@ -819,19 +888,43 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         os.chdir(opts.chdir)
 
     dbus_pid, dbus_env = 0, {}
-    if not shadowing and POSIX and not OSX and not clobber:
-        assert starting or starting_desktop or proxying
-        try:
-            from xpra.server.dbus.dbus_start import start_dbus
-        except ImportError as e:
-            log("dbus components are not installed: %s", e)
-        else:
-            dbus_pid, dbus_env = start_dbus(opts.dbus_launch)
-            if dbus_env:
-                os.environ.update(dbus_env)
+    if not shadowing and POSIX and not OSX:
+        dbuslog = Logger("dbus")
+        if not start_vfb:
+            #try to retrieve it:
+            from xpra.scripts.main import exec_wminfo
+            wminfo = exec_wminfo(display_name)
+            try:
+                dbus_pid = int(wminfo.get("dbus-pid", 0))
+            except ValueError:
+                pass
+            dbus_address = wminfo.get("dbus-address")
+            if dbus_pid and os.path.exists("/proc") and not os.path.exists("/proc/%s" % dbus_pid):
+                dbuslog("dbus pid %s is no longer valid")
+                dbus_pid = 0
+            if dbus_pid and dbus_address:
+                dbus_env["DBUS_SESSION_BUS_PID"] = str(dbus_pid)
+                dbus_env["DBUS_SESSION_BUS_ADDRESS"] = dbus_address
+                dbus_window = wminfo.get("dbus-window")
+                if dbus_window:
+                    dbus_env["DBUS_SESSION_BUS_WINDOW_ID"] = dbus_window
+                dbuslog("retrieved dbus environment: %s", dbus_env)
+        if not dbus_env:
+            no_gtk()
+            assert starting or starting_desktop or proxying
+            try:
+                from xpra.server.dbus.dbus_start import start_dbus
+            except ImportError as e:
+                dbuslog("dbus components are not installed: %s", e)
+            else:
+                dbus_pid, dbus_env = start_dbus(opts.dbus_launch)
+                if dbus_env:
+                    dbuslog("started new dbus instance: %s", dbus_env)
+                    os.environ.update(dbus_env)
 
     if not proxying:
         if POSIX and not OSX:
+            no_gtk()
             if starting or starting_desktop or shadowing:
                 r = verify_display(xvfb, display_name, shadowing)
                 if r:
@@ -841,33 +934,34 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         log("gui_init()")
         gui_init()
 
-    progress(50, "creating local sockets")
-    #setup unix domain socket:
-    netlog = get_network_logger()
-    local_sockets = setup_local_sockets(opts.bind,
-                                        opts.socket_dir, opts.socket_dirs,
-                                        display_name, clobber,
-                                        opts.mmap_group, opts.socket_permissions,
-                                        username, uid, gid)
-    netlog("setting up local sockets: %s", local_sockets)
-    sockets.update(local_sockets)
-    if POSIX and (starting or upgrading or starting_desktop or upgrading_desktop):
-        #all unix domain sockets:
-        ud_paths = [sockpath for stype, _, sockpath, _ in local_sockets if stype=="unix-domain"]
-        if ud_paths:
-            #choose one so our xdg-open override script can use to talk back to us:
-            if opts.forward_xdg_open:
-                for x in ("/usr/libexec/xpra", "/usr/lib/xpra"):
-                    xdg_override = os.path.join(x, "xdg-open")
-                    if os.path.exists(xdg_override):
-                        os.environ["PATH"] = x+os.pathsep+os.environ.get("PATH", "")
-                        os.environ["XPRA_SERVER_SOCKET"] = ud_paths[0]
-                        break
-        else:
-            log.warn("Warning: no local server sockets,")
-            if opts.forward_xdg_open:
-                log.warn(" forward-xdg-open cannot be enabled")
-            log.warn(" non-embedded ssh connections will not be available")
+    def init_local_sockets():
+        progress(60, "initializing local sockets")
+        #setup unix domain socket:
+        netlog = get_network_logger()
+        local_sockets = setup_local_sockets(opts.bind,
+                                            opts.socket_dir, opts.socket_dirs,
+                                            display_name, clobber,
+                                            opts.mmap_group, opts.socket_permissions,
+                                            username, uid, gid)
+        netlog("setting up local sockets: %s", local_sockets)
+        sockets.update(local_sockets)
+        if POSIX and (starting or upgrading or starting_desktop or upgrading_desktop):
+            #all unix domain sockets:
+            ud_paths = [sockpath for stype, _, sockpath, _ in local_sockets if stype=="unix-domain"]
+            if ud_paths:
+                #choose one so our xdg-open override script can use to talk back to us:
+                if opts.forward_xdg_open:
+                    for x in ("/usr/libexec/xpra", "/usr/lib/xpra"):
+                        xdg_override = os.path.join(x, "xdg-open")
+                        if os.path.exists(xdg_override):
+                            os.environ["PATH"] = x+os.pathsep+os.environ.get("PATH", "")
+                            os.environ["XPRA_SERVER_SOCKET"] = ud_paths[0]
+                            break
+            else:
+                log.warn("Warning: no local server sockets,")
+                if opts.forward_xdg_open:
+                    log.warn(" forward-xdg-open cannot be enabled")
+                log.warn(" non-embedded ssh connections will not be available")
 
     set_server_features(opts)
 
@@ -882,7 +976,7 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         if uinput_uuid:
             save_uinput_id(uinput_uuid)
 
-    progress(60, "initializing server")
+    progress(40, "initializing server")
     if shadowing:
         app = make_shadow_server()
     elif proxying:
@@ -892,18 +986,21 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
             app = make_server(clobber)
         else:
             assert starting_desktop or upgrading_desktop
-            app = make_desktop_server(clobber)
+            app = make_desktop_server()
         app.init_virtual_devices(devices)
 
     try:
         app.exec_cwd = opts.chdir or cwd
         app.display_name = display_name
         app.init(opts)
-        progress(70, "initializing sockets")
+        if not app.validate():
+            return 1
+        init_local_sockets()
         app.init_sockets(sockets)
         app.init_dbus(dbus_pid, dbus_env)
         if not shadowing and not proxying:
             app.init_display_pid(xvfb_pid)
+            app.save_pid()
         app.original_desktop_display = desktop_display
         del opts
         if not app.server_ready():
