@@ -98,7 +98,7 @@ class ServerBaseControlCommands(StubServerMixin):
             ArgsControlCommand("scaling",               "set a specific scaling value",     min_args=1, validation=[parse_scaling_value]),
             ArgsControlCommand("auto-refresh",          "set a specific auto-refresh value", min_args=1, validation=[float]),
             ArgsControlCommand("refresh",               "refresh some or all windows",      min_args=0),
-            ArgsControlCommand("encoding",              "picture encoding",                 min_args=1, max_args=1),
+            ArgsControlCommand("encoding",              "picture encoding",                 min_args=2),
             ArgsControlCommand("video-region-enabled",  "enable video region",              min_args=2, max_args=2, validation=[int, parse_boolean_value]),
             ArgsControlCommand("video-region-detection","enable video detection",           min_args=2, max_args=2, validation=[int, parse_boolean_value]),
             ArgsControlCommand("video-region-exclusion-zones","set window regions to exclude from video regions: 'WID,(x,y,w,h),(x,y,w,h),..', ie: '1 (0,10,100,20),(200,300,20,20)'",  min_args=2, max_args=2, validation=[int, parse_4intlist]),
@@ -290,7 +290,7 @@ class ServerBaseControlCommands(StubServerMixin):
         #so we may have to do an ugly roundtrip:
         try:
             filename = strtobytes(filename).decode("utf8")
-        except:
+        except Exception:
             filename = bytestostr(filename)
         openit = str(openit).lower() in ("open", "true", "1")
         return self.do_control_file_command("send file", client_uuids, filename, "file_transfer", (False, openit))
@@ -436,7 +436,7 @@ class ServerBaseControlCommands(StubServerMixin):
                 try:
                     wid = int(x)
                 except:
-                    raise ControlError("invalid window id: %s" % x)
+                    raise ControlError("invalid window id: %s" % x) from None
                 if wid in self._id_to_window:
                     wids.append(wid)
                 else:
@@ -607,7 +607,7 @@ class ServerBaseControlCommands(StubServerMixin):
                 keycode = int(keycode_str)
             assert keycode>0 and keycode<=255
         except:
-            raise ControlError("invalid keycode specified: '%s' (must be a number between 1 and 255)" % keycode_str)
+            raise ControlError("invalid keycode specified: '%s' (must be a number between 1 and 255)" % keycode_str) from None
         if press is not True:
             if press in ("1", "press"):
                 press = True
