@@ -274,9 +274,10 @@ class ProxyServer(ServerCore):
         generic_request = c.strget("request")
         def is_req(mode):
             return generic_request==mode or c.boolget("%s_request" % mode)
-        if any(is_req(x) for x in ("screenshot", "event", "print", "exit")):
-            self.send_disconnect(proto, "invalid request")
-            return
+        for x in ("screenshot", "event", "print", "exit"):
+            if is_req(x):
+                self.send_disconnect(proto, "error: invalid request, '%s' is not supported by the proxy server" % x)
+                return
         if is_req("stop"):
             #global kill switch:
             if not CAN_STOP_PROXY:
@@ -650,12 +651,14 @@ class ProxyServer(ServerCore):
 
 
     def get_info(self, proto, *_args):
-        info = ServerCore.get_server_info(self)
-        info.setdefault("server", {})["type"] = "Python/GLib/proxy"
-        #only show more info if we have authenticated
-        #as the user running the proxy server process:
-        if proto and proto.authenticators:
-            sessions = []
+        authenticated = proto and proto.authenticators
+        if not authenticated:
+            info = super().get_server_info()
+        else:
+            #only show more info if we have authenticated
+            #as the user running the proxy server process:
+            info = super().get_info(proto)
+            sessions = ()
             for authenticator in proto.authenticators:
                 auth_sessions = authenticator.get_sessions()
                 if auth_sessions:
@@ -684,4 +687,5 @@ class ProxyServer(ServerCore):
                         i += 1
                     info["instances"] = instances_info
                     info["proxies"] = len(instances)
+        info.setdefault("server", {})["type"] = "Python/GLib/proxy"
         return info
