@@ -13,7 +13,6 @@ from xpra.util import updict, log_screen_sizes, envbool, csv
 from xpra.platform.paths import get_icon, get_icon_filename
 from xpra.platform.gui import get_wm_name
 from xpra.server import server_features
-from xpra.server.source.windows_mixin import WindowsMixin
 from xpra.gtk_common.gobject_util import one_arg_signal, no_arg_signal
 from xpra.gtk_common.error import XError
 from xpra.gtk_common.gtk_util import get_screen_sizes, get_root_size
@@ -39,7 +38,6 @@ RandR = RandRBindings()
 
 windowlog = Logger("server", "window")
 geomlog = Logger("server", "window", "geometry")
-settingslog = Logger("x11", "xsettings")
 metadatalog = Logger("x11", "metadata")
 screenlog = Logger("screen")
 iconlog = Logger("icon")
@@ -275,19 +273,19 @@ DesktopServerBaseClass = type('DesktopServerBaseClass', DESKTOPSERVER_BASES, {})
 log("DesktopServerBaseClass%s", DESKTOPSERVER_BASES)
 
 
-"""
-    A server class for RFB / VNC-like desktop displays,
-    used with the "start-desktop" subcommand.
-"""
 class XpraDesktopServer(DesktopServerBaseClass):
+    """
+        A server class for RFB / VNC-like desktop displays,
+        used with the "start-desktop" subcommand.
+    """
     __gsignals__ = {
         "xpra-xkb-event"        : one_arg_signal,
         "xpra-cursor-event"     : one_arg_signal,
         "xpra-motion-event"     : one_arg_signal,
         }
 
-    def __init__(self, clobber=False):
-        X11ServerBase.__init__(self, clobber)
+    def __init__(self):
+        X11ServerBase.__init__(self)
         for c in DESKTOPSERVER_BASES:
             if c!=X11ServerBase:
                 c.__init__(self)
@@ -450,7 +448,7 @@ class XpraDesktopServer(DesktopServerBaseClass):
 
 
     def get_server_mode(self):
-        return "GTK3 X11 desktop"
+        return "X11 desktop"
 
     def make_hello(self, source):
         capabilities = super().make_hello(source)
@@ -489,10 +487,9 @@ class XpraDesktopServer(DesktopServerBaseClass):
         wid = self._window_to_id[window]
         x, y, w, h = window.get_geometry()
         geomlog("window_resized_signaled(%s) geometry=%s", window, (x, y, w, h))
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                ss.resize_window(wid, window, w, h)
-                ss.damage(wid, window, 0, 0, w, h)
+        for ss in self.window_sources():
+            ss.resize_window(wid, window, w, h)
+            ss.damage(wid, window, 0, 0, w, h)
 
 
     def send_initial_windows(self, ss, sharing=False):
