@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,7 +9,8 @@ from gi.repository import GObject, Gtk, Gdk
 from xpra.util import envint, envbool, typedict
 from xpra.common import MAX_WINDOW_SIZE
 from xpra.gtk_common.gobject_util import one_arg_signal, non_none_list_accumulator, SIGNAL_RUN_LAST
-from xpra.gtk_common.error import XError, XSwallowContext
+from xpra.gtk_common.error import XError, XSwallowContext, xlog
+from xpra.x11.gtk_x11 import GDKX11Window
 from xpra.x11.gtk_x11.send_wm import send_wm_take_focus
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
 from xpra.x11.prop_conv import MotifWMHints
@@ -24,10 +25,8 @@ from xpra.x11.gtk_x11.gdk_bindings import (
     calc_constrained_size,
     x11_get_server_time,
     )
-from xpra.gtk_common.gtk_util import (
-    get_default_root_window,
-    GDKWindow,
-    )
+
+from xpra.gtk_common.gtk_util import get_default_root_window
 from xpra.log import Logger
 
 log = Logger("x11", "window")
@@ -176,12 +175,14 @@ class WindowModel(BaseWindowModel):
     def setup(self):
         super().setup()
 
-        ox, oy, ow, oh = self.client_window.get_geometry()[:4]
+        ogeom = self.client_window.get_geometry()
+        ox, oy, ow, oh = ogeom[:4]
         # We enable PROPERTY_CHANGE_MASK so that we can call
         # x11_get_server_time on this window.
         # clamp this window to the desktop size:
         x, y = self._clamp_to_desktop(ox, oy, ow, oh)
-        self.corral_window = GDKWindow(self.parking_window,
+        geomlog("setup() clamp_to_desktop(%s)=%s", ogeom, (x, y))
+        self.corral_window = GDKX11Window(self.parking_window,
                                         x=x, y=y, width=ow, height=oh,
                                         window_type=Gdk.WindowType.CHILD,
                                         event_mask=Gdk.EventMask.PROPERTY_CHANGE_MASK,
@@ -209,20 +210,35 @@ class WindowModel(BaseWindowModel):
         X11Window.Reparent(self.xid, cxid, 0, 0)
         self.client_reparented = True
 
-        geomlog("setup() geometry")
         geom = X11Window.geometry_with_border(self.xid)
         if geom is None:
             raise Unmanageable("window %#x disappeared already" % self.xid)
-        w, h = geom[2:4]
+        geomlog("setup() geometry=%s, ogeom=%s", geom, ogeom)
+        nx, ny, w, h = geom[:4]
+        #after reparenting, the coordinates of the client window should be 0,0
+        #use the coordinates of the corral window:
+        if nx==ny==0:
+            nx, ny = x, y
         hints = self.get_property("size-hints")
         geomlog("setup() hints=%s size=%ix%i", hints, w, h)
         nw, nh = self.calc_constrained_size(w, h, hints)
-        self._updateprop("geometry", (x, y, nw, nh))
-        geomlog("setup() resizing windows to %sx%s", nw, nh)
-        #don't trigger a resize unless we have to:
-        if ow!=nw or oh!=nh:
+        pos = hints.get("position")
+        if pos==(0, 0) and (nx!=0 or ny!=0):
+            #never override with 0,0
+            hints.pop("position")
+            pos = None
+        if ox==0 and oy==0 and pos:
+            nx, ny = pos
+        self._updateprop("geometry", (nx, ny, nw, nh))
+        geomlog("setup() resizing windows to %sx%s, moving to %i,%i", nw, nh, nx, ny)
+        #don't trigger a move or resize unless we have to:
+        if (ox,oy)!=(nx,ny) and (ow,oh)!=(nw,nh):
+            self.corral_window.move_resize(nx, ny, nw, nh)
+        elif (ox,oy)!=(nx,ny):
+            self.corral_window.move(nx, ny)
+        elif (ow,oh)!=(nw,nh):
             self.corral_window.resize(nw, nh)
-        if w!=nw or h!=nh:
+        if (ow,oh)!=(nw,nh):
             self.client_window.resize(nw, nh)
         self.client_window.show_unraised()
         #this is here to trigger X11 errors if any are pending
@@ -293,7 +309,11 @@ class WindowModel(BaseWindowModel):
         #but allow size hints to override it if specified
         x, y, w, h = geom[:4]
         size_hints = self.get_property("size-hints")
-        ax, ay = size_hints.get("position", (x, y))
+        ax, ay = size_hints.get("position", (0, 0))
+        if ax==ay==0 and (x!=0 or y!=0):
+            #don't override with 0,0
+            size_hints.pop("position", None)
+            ax, ay = x, y
         aw, ah = size_hints.get("size", (w, h))
         geomlog("initial X11 position and size: requested(%s, %s, %s)=%s",
                 (x, y, w, h), size_hints, geom, (ax, ay, aw, ah))
@@ -302,7 +322,8 @@ class WindowModel(BaseWindowModel):
         set_if_unset("requested-size", (aw, ah))
         #it may have been set already:
         v = self.get_property("set-initial-position")
-        self._internal_set_property("set-initial-position", v or ("position" in size_hints))
+        if not v and ("position" in size_hints):
+            self._internal_set_property("set-initial-position", True)
         self.update_children()
 
     def do_unmanaged(self, wm_exiting):
@@ -809,12 +830,15 @@ class WindowModel(BaseWindowModel):
     def give_client_focus(self):
         """The focus manager has decided that our client should receive X
         focus.  See world_window.py for details."""
+        log("give_client_focus() corral_window=%s", self.corral_window)
         if self.corral_window:
-            with xswallow:
+            with xlog:
                 self.do_give_client_focus()
 
     def do_give_client_focus(self):
-        focuslog("Giving focus to %#x", self.xid)
+        protocols = self.get_property("protocols")
+        focuslog("Giving focus to %#x, input_field=%s, FORCE_XSETINPUTFOCUS=%s, protocols=%s",
+                 self.xid, self._input_field, FORCE_XSETINPUTFOCUS, protocols)
         # Have to fetch the time, not just use CurrentTime, both because ICCCM
         # says that WM_TAKE_FOCUS must use a real time and because there are
         # genuine race conditions here (e.g. suppose the client does not
@@ -837,7 +861,7 @@ class WindowModel(BaseWindowModel):
         if bool(self._input_field) or FORCE_XSETINPUTFOCUS:
             focuslog("... using XSetInputFocus")
             X11Window.XSetInputFocus(self.xid, now)
-        if "WM_TAKE_FOCUS" in self.get_property("protocols"):
+        if "WM_TAKE_FOCUS" in protocols:
             focuslog("... using WM_TAKE_FOCUS")
             send_wm_take_focus(self.client_window, now)
         self.set_active()
