@@ -17,12 +17,12 @@ from xpra.util import updict, rindex, envbool, envint, typedict, AdHocStruct, WO
 from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr, monotonic_time
 from xpra.common import CLOBBER_UPGRADE, MAX_WINDOW_SIZE
 from xpra.server import server_features
-from xpra.server.source.windows_mixin import WindowsMixin
 from xpra.gtk_common.gobject_util import one_arg_signal
 from xpra.gtk_common.gtk_util import get_default_root_window, get_pixbuf_from_data
 from xpra.x11.common import Unmanageable
 from xpra.x11.gtk_x11.prop import prop_set
 from xpra.x11.gtk_x11.tray import get_tray_window, SystemTray
+from xpra.x11.gtk_x11.selection import AlreadyOwned
 from xpra.x11.gtk_x11.gdk_bindings import (
     add_event_receiver,
     get_children,
@@ -185,6 +185,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
         }
 
     def __init__(self, clobber):
+        self.clobber = clobber
         self.root_overlay = None
         self.repaint_root_overlay_timer = None
         self.configure_damage_timers = {}
@@ -195,8 +196,9 @@ class XpraServer(GObject.GObject, X11ServerBase):
         self.last_raised = None
         self.system_tray = False
         GObject.GObject.__init__(self)
-        X11ServerBase.__init__(self, clobber)
+        X11ServerBase.__init__(self)
         self.session_type = "seamless"
+        self._xsettings_enabled = True
 
     def init(self, opts):
         self.wm_name = opts.wm_name
@@ -217,7 +219,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
             with xlog:
                 set_initial_resolution(self.initial_resolution or DEFAULT_VFB_RESOLUTION)
 
-    def server_ready(self):
+    def validate(self):
         if not X11Window.displayHasXComposite():
             log.error("Xpra 'start' subcommand runs as a compositing manager")
             log.error(" it cannot use a display which lacks the XComposite extension!")
@@ -266,7 +268,16 @@ class XpraServer(GObject.GObject, X11ServerBase):
         ### Create the WM object
         with xsync:
             from xpra.x11.gtk_x11.wm import Wm
-            self._wm = Wm(self.clobber, self.wm_name)
+            try:
+                self._wm = Wm(self.clobber, self.wm_name)
+            except AlreadyOwned:
+                log("Error: cannot create our window manager", exc_info=True)
+                display = os.environ.get("DISPLAY", "")
+                #make sure we don't kill the vfb since we don't own it:
+                from xpra.server import EXITING_CODE
+                self._upgrading = EXITING_CODE
+                from xpra.scripts.config import InitException  #pylint: disable=import-outside-toplevel
+                raise InitException("another window manager is active on display '%s'" % display) from None
         if server_features.windows:
             self._wm.connect("new-window", self._new_window_signaled)
         self._wm.connect("quit", lambda _: self.clean_quit(True))
@@ -322,7 +333,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
 
 
     def get_server_mode(self):
-        return "GTK3 X11"
+        return "X11 seamless"
 
 
     def server_event(self, *args):
@@ -483,7 +494,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
     def parse_hello_ui_window_settings(self, ss, _caps):
         #FIXME: with multiple users, don't set any frame size?
         frame = None
-        if isinstance(ss, WindowsMixin):
+        if ss in self.window_sources():
             window_frame_sizes = ss.window_frame_sizes
             framelog("parse_hello_ui_window_settings: client window_frame_sizes=%s", window_frame_sizes)
             if window_frame_sizes:
@@ -567,9 +578,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
         #name, dtype, dformat, value = event
         metadata = {"x11-property" : event}
         wid = self._window_to_id[window]
-        for ss in self._server_sources.values():
-            if not isinstance(ss, WindowsMixin):
-                continue
+        for ss in self.window_sources():
             ms = getattr(ss, "metadata_supported", ())
             if "x11-property" in ms:
                 ss.send("window-metadata", wid, metadata)
@@ -619,8 +628,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
             return
         x, y, nw, nh = self._desktop_manager.window_geometry(window)
         resize_counter = self._desktop_manager.get_resize_counter(window, 1)
-        wsources = [ss for ss in self._server_sources.values() if isinstance(ss, WindowsMixin)]
-        for ss in wsources:
+        for ss in self.window_sources():
             ss.move_resize_window(wid, window, x, y, nw, nh, resize_counter)
             #refresh to ensure the client gets the new window contents:
             #TODO: to save bandwidth, we should compare the dimensions and skip the refresh
@@ -696,9 +704,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
             return
         geomlog("or_window_geometry_changed: %s (window=%s)", geom, window)
         wid = self._window_to_id[window]
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                ss.or_window_geometry(wid, window, x, y, w, h)
+        for ss in self.window_sources():
+            ss.or_window_geometry(wid, window, x, y, w, h)
 
 
     def add_control_commands(self):
@@ -718,9 +725,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
 
     def _show_desktop(self, wm, show):
         log("show_desktop(%s, %s)", wm, show)
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                ss.show_desktop(show)
+        for ss in self.window_sources():
+            ss.show_desktop(show)
 
 
     def _focus(self, server_source, wid, modifiers):
@@ -761,8 +767,9 @@ class XpraServer(GObject.GObject, X11ServerBase):
             return
         focuslog("focus: giving focus to %s", window)
         with xswallow:
+            self.last_raised = wid
             window.raise_window()
-        window.give_client_focus()
+            window.give_client_focus()
         if server_source and modifiers is not None:
             make_keymask_match = getattr(server_source, "make_keymask_match", None)
             if make_keymask_match:
@@ -785,9 +792,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
 
     def _send_new_tray_window_packet(self, wid, window):
         ww, wh = window.get_dimensions()
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                ss.new_tray(wid, window, ww, wh)
+        for ss in self.window_sources():
+            ss.new_tray(wid, window, ww, wh)
         self.refresh_window(window)
 
 
@@ -809,9 +815,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
         if grab_id<0 or self._has_grab==grab_id:
             return
         self._has_grab = grab_id
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                ss.pointer_grab(self._has_grab)
+        for ss in self.window_sources():
+            ss.pointer_grab(self._has_grab)
 
     def _window_ungrab(self, window, event):
         grab_id = self._window_to_id.get(window, -1)
@@ -820,9 +825,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
         if not self._has_grab:
             return
         self._has_grab = 0
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                ss.pointer_ungrab(grab_id)
+        for ss in self.window_sources():
+            ss.pointer_ungrab(grab_id)
 
 
     def _initiate_moveresize(self, window, event):
@@ -831,7 +835,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
         #x_root, y_root, direction, button, source_indication = event.data
         wid = self._window_to_id[window]
         #find clients that handle windows:
-        wsources = [ss for ss in self._server_sources.values() if isinstance(ss, WindowsMixin)]
+        wsources = self.window_sources()
         if not wsources:
             return
         #prefer the "UI driver" if we find it:
@@ -851,9 +855,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
             self.last_raised = None
         if detail==0 and self._has_focus==wid:  #Above=0
             return
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                ss.restack_window(wid, window, detail, sibling)
+        for ss in self.window_sources():
+            ss.restack_window(wid, window, detail, sibling)
 
 
     def _set_window_state(self, proto, wid, window, new_window_state):
@@ -949,9 +952,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
             self._set_window_state(proto, wid, window, packet[3])
         if self._desktop_manager.is_shown(window):
             geomlog("client %s unmapped window %s - %s", ss, wid, window)
-            for ss in self._server_sources.values():
-                if isinstance(ss, WindowsMixin):
-                    ss.unmap_window(wid, window)
+            for ss in self.window_sources():
+                ss.unmap_window(wid, window)
             window.unmap()
             iconified = len(packet)>=3 and bool(packet[2])
             if iconified and not window.get_property("iconic"):
@@ -1042,8 +1044,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
                     if resized and SHARING_SYNC_SIZE:
                         #try to ensure this won't trigger a resizing loop:
                         counter = max(0, resize_counter-1)
-                        for s in self._server_sources.values():
-                            if s!=ss and isinstance(s, WindowsMixin):
+                        for s in self.window_sources():
+                            if s!=ss:
                                 s.resize_window(wid, window, aw, ah, resize_counter=counter)
                     damage |= owx!=ax or owy!=ay or resized
             if not shown and not skip_geometry:
