@@ -8,6 +8,7 @@
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000041.html
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000042.html
 # (also do not import anything that imports gtk)
+import signal
 from subprocess import Popen, PIPE, call
 import os.path
 
@@ -21,6 +22,7 @@ from xpra.os_util import (
     POSIX, OSX,
     )
 from xpra.platform.displayfd import read_displayfd, parse_displayfd
+from xpra.log import Logger
 
 
 VFB_WAIT = envint("XPRA_VFB_WAIT", 3)
@@ -56,7 +58,6 @@ vfb_logger = None
 def get_vfb_logger():
     global vfb_logger
     if not vfb_logger:
-        from xpra.log import Logger
         vfb_logger = Logger("server", "x11", "screen")
     return vfb_logger
 
@@ -153,7 +154,7 @@ def get_xauthority_path(display_name, username, uid, gid):
         filename = ".Xauthority"
     return os.path.join(pathexpand(d), filename)
 
-def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, xauth_data, uinput_uuid=None):
+def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, uinput_uuid=None):
     if not POSIX:
         raise InitException("starting an Xvfb is not supported on %s" % os.name)
     if OSX:
@@ -163,21 +164,7 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
 
     cleanups = []
     log = get_vfb_logger()
-    log("start_Xvfb%s", (xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, xauth_data, uinput_uuid))
-    xauthority = get_xauthority_path(display_name, username, uid, gid)
-    os.environ["XAUTHORITY"] = xauthority
-    if not os.path.exists(xauthority):
-        log("creating XAUTHORITY=%s with data=%s", xauthority, xauth_data)
-        try:
-            with open(xauthority, "a") as f:
-                if getuid()==0 and (uid!=0 or gid!=0):
-                    os.fchown(f.fileno(), uid, gid)
-        except Exception as e:
-            #trying to continue anyway!
-            log.error("Error trying to create XAUTHORITY file %s:", xauthority)
-            log.error(" %s", e)
-    else:
-        log("found existing XAUTHORITY file '%s'", xauthority)
+    log("start_Xvfb%s", (xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, uinput_uuid))
     use_display_fd = display_name[0]=='S'
 
     subs = {}
@@ -323,8 +310,6 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
             log("xvfb_cmd=%s", xvfb_cmd)
             xvfb = Popen(xvfb_cmd, executable=xvfb_executable,
                          stdin=PIPE, preexec_fn=preexec)
-
-        xauth_add(xauthority, display_name, xauth_data, uid, gid)
     except Exception as e:
         if xvfb and xvfb.poll() is None:
             log.error(" stopping vfb process with pid %i", xvfb.pid)
@@ -338,7 +323,6 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
 def kill_xvfb(xvfb_pid):
     log = get_vfb_logger()
     log.info("killing xvfb with pid %s", xvfb_pid)
-    import signal
     try:
         os.kill(xvfb_pid, signal.SIGTERM)
     except OSError as e:
@@ -386,16 +370,18 @@ def xauth_add(filename, display_name, xauth_data, uid, gid):
                 setuidgid(uid, gid)
         xauth_cmd = ["xauth"]+xauth_args
         start = monotonic_time()
+        log = get_vfb_logger()
+        log("xauth command: %s", xauth_cmd)
         code = call(xauth_cmd, preexec_fn=preexec)
         end = monotonic_time()
         if code!=0 and (end-start>=10):
-            log = get_vfb_logger()
             log.warn("Warning: xauth command took %i seconds and failed" % (end-start))
             #took more than 10 seconds to fail, check for stale locks:
             import glob
             if glob.glob("%s-*" % filename):
                 log.warn("Warning: trying to clean some stale xauth locks")
                 xauth_cmd = ["xauth", "-b"]+xauth_args
+                log("xauth command: %s", xauth_cmd)
                 code = call(xauth_cmd, preexec_fn=preexec)
         if code!=0:
             raise OSError("non-zero exit code: %s" % code)
