@@ -17,7 +17,6 @@ from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImpo
 from xpra.gtk_common.error import XError, xswallow, xsync, xlog, trap, verify_sync
 from xpra.gtk_common.gtk_util import get_default_root_window
 from xpra.server.server_uuid import save_uuid, get_uuid
-from xpra.server.source.windows_mixin import WindowsMixin
 from xpra.x11.vfb_util import parse_resolution
 from xpra.x11.fakeXinerama import find_libfakeXinerama, save_fakeXinerama_config, cleanup_fakeXinerama
 from xpra.x11.gtk_x11.prop import prop_get, prop_set
@@ -144,6 +143,11 @@ class X11ServerCore(GTKServerBase):
         with xlog:
             self.x11_filter = init_x11_filter()
         assert self.x11_filter
+        with xlog:
+            self.save_mode()
+
+    def save_mode(self):
+        prop_set(get_default_root_window(), "XPRA_SERVER_MODE", "latin1", self.get_server_mode())
 
     def init_fake_xinerama(self):
         if self.fake_xinerama in FALSE_OPTIONS:
@@ -340,7 +344,7 @@ class X11ServerCore(GTKServerBase):
             "fakeXinerama"          : bool(self.libfakeXinerama_so),
             "libfakeXinerama"       : self.libfakeXinerama_so or "",
             })
-        log("X11ServerBase.do_get_info took %ims", (monotonic_time()-start)*1000)
+        log("X11ServerCore.do_get_info took %ims", (monotonic_time()-start)*1000)
         return info
 
     def get_ui_info(self, proto, wids=None, *args) -> dict:
@@ -788,9 +792,8 @@ class X11ServerCore(GTKServerBase):
             return
         cursorlog("cursor_event: %s", event)
         self.last_cursor_serial = event.cursor_serial
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                ss.send_cursor()
+        for ss in self.window_sources():
+            ss.send_cursor()
 
 
     def _motion_signaled(self, model, event):
@@ -815,10 +818,9 @@ class X11ServerCore(GTKServerBase):
         #this method is a catch-all for events on windows we don't manage,
         #so we use wid=0 for that:
         wid = 0
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                name = strtobytes(event.bell_name or "")
-                ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, name)
+        for ss in self.window_sources():
+            name = strtobytes(event.bell_name or "")
+            ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, name)
 
 
     def _bell_signaled(self, wm, event):
@@ -829,10 +831,9 @@ class X11ServerCore(GTKServerBase):
         if event.window!=get_default_root_window() and event.window_model is not None:
             wid = self._window_to_id.get(event.window_model, 0)
         log("_bell_signaled(%s,%r) wid=%s", wm, event, wid)
-        for ss in self._server_sources.values():
-            if isinstance(ss, WindowsMixin):
-                name = strtobytes(event.bell_name or "")
-                ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, name)
+        for ss in self.window_sources():
+            name = strtobytes(event.bell_name or "")
+            ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, name)
 
 
     def get_screen_number(self, _wid):
