@@ -90,7 +90,7 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant, xkbmap_options,
         if layout:
             log.info("setting keymap: %s",
                      csv("%s=%s" % (std(k), std(v)) for k,v in xkbmap_query_struct.items()
-                         if k in ("rules", "model", "layout") and v))
+                         if bytestostr(k) in ("rules", "model", "layout", "variant", "options") and v))
             safe_setxkbmap(rules, model, layout, variant, options)
         else:
             safe_setxkbmap(rules, model, "", "", "")
